/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#ifndef __EPP_EPPXMLBASE_H
#define __EPP_EPPXMLBASE_H

/** @file epp_eppXMLbase.h
 *  @brief Declaration of XML helper-functions for general use
 *  @author Asbjorn Steira Mikkelsen
 */

#include <domtools/dom_wrapper.h>
#include "../config.h"
#include "epp_eppdata.h"
#include "epp_AuthInfo.h"
#include "epp_DataCollectionPolicy.h"
#include "epp_Response.h"
#include "epp_Greeting.h"
#include "epp_Service.h"
#include "../epp_Unspec.h"
#include "../epp_Extension.h"


namespace eppobject { namespace epp {

   /** Writes a standard EPP XML header to the output object
    * @param outputobject The output object to write to
    */
   void createStandardXMLHeader(EPP_output & outputobject);
   
   
   /** Writes a standard EPP XML footer to the output object
    * @param outputobject The output object to write to
    */
   void createStandardXMLFooter(EPP_output & outputobject);   
   
   
   /** Writes a given epp_Extension element to the output object
    * @param outputobject The output object to write to
    * @param unspecpart The epp_Extension element to add
    */
   void addExtensionElement(EPP_output & outputobject,
			    const epp_Extension_ref & unspecpart);
   
   
   /** Writes a given epp_Extension elements to the output object
    * @param outputobject The output object to write to
    * @param unspecparts The epp_Extension elements to add
    */
   void addExtensionElements(EPP_output & outputobject,
			     const epp_extension_ref_seq_ref & extensions);
   
   
   /** Writes a given epp_AuthInfo element to the output-object
    * @param outputobject The output object to write to
    * @param element The epp_AuthInfo element to add
    * @param tag The encapsulating XML-tag to use
    * @throw epp_XMLException If mandatory data is missing in the epp_AuthInfo object
    */
   void addAuthInfoElement(EPP_output & outputobject,
			   const epp_AuthInfo_ref & element,
			   const epp_string & tag);


   /** Parses an authInfo dom_ptr and creates the corresponding epp_AuthInfo object
    * @param authNode The authInfo dom_ptr
    * @return The corresponding epp_AuthInfo object
    */
   epp_AuthInfo_ref createAuthInfoObject(const domtools::dom_ptr & authNode);
   
   
   /** Returns the corresponsing epp_string text for the given epp_TransferOpType-enum
    * @param statusType The transfer op-type as an epp_TransferOpType-enum
    * @return The transfer op-type as an epp_string
    * @throw epp_XMLException If statusType is not a valid epp_TransferOpType
    */
   epp_string returnTransferType(const epp_TransferOpType & statusType);

   
   /** Returns the corresponsing epp_TransferOpType for the given epp_string
    * @param statusType The transfer op-type as an epp_string
    * @return The transfer op-type as an epp_TransferOpType-enum
    * @throw epp_XMLException If statusType is not a valid epp_TransferOpType
    */
   epp_TransferOpType returnTransferType(const epp_string & statusType);
   
   
   /** Returns the corresponsing epp_TransferStatusType for a given epp_string
    * @param statusType The transfer status type as an epp_string
    * @return The transfer status type as an epp_TransferStatusType-enum
    * @throw epp_XMLException If statusType is not a valid epp_TransferStatusType
    */
   epp_TransferStatusType returnTransferStatusType(const epp_string & statusType);
   

   /** Returns the corresponsing epp_string for a given epp_TransferStatusType
    * @param statusType The transfer status type as an epp_TransferStatusType
    * @return The transfer status type as an epp_string
    * @throw epp_XMLException If statusType is not a valid epp_TransferStatusType
    */
   epp_string returnTransferStatusType(const epp_TransferStatusType & statusType);

   /** Returns the corresponsing epp_AuthInfoType for a given epp_string
    * @param infoType The authInfo type as an epp_string
    * @return The authInfo type as an epp_AuthInfoType-enum
    * @throw epp_XMLException If infoType is not a valid epp_AuthInfoType
    */
   epp_AuthInfoType returnAuthInfoType(const epp_string & infoType);

   
   /** Parses an XML epp_string and generates a DOMDocument of it
    * @param xml The XML epp_string to build a tree from
    * @return The generated DOM-tree, as a dom_ptr
    * @throw epp_XMLException If it is unable to initialise the XML-parser
    * @throw epp_XMLException If it is unable to parse the epp_string
    * @todo Change so it is not depending on using Xerces at all
    */
   domtools::dom_ptr createDOMDocument(const epp_string & xml);
   

   /** Parses a response dom_ptr and creates the corresponding epp_Response object.
    * "resData" will not be handled here, as it will be different for different types
    * of requests
    * @param node The response dom_ptr
    * @return The corresponding epp_Response object
    * @throw epp_XMLException If it is unable find the correct XML-elements in the Node
    * @throw epp_Exception If there response node reports an error condition
    */
   epp_Response_ref createGenericResponse(const domtools::dom_ptr & node);
   
   
   /** Parses a greeting dom_ptr and creates the corresponding epp_Greeting object
    * @param node The greeting dom_ptr
    * @return The corresponding epp_Greeting object
    * @throw epp_XMLException If it is unable find the correct XML-elements in the Node
    */
   epp_Greeting_ref createGreetingResponse(const domtools::dom_ptr & node);

   
   /** Parses a dom_ptr and returns the requested resData as a dom_ptr
    * @param responseNode The dom_ptr to investigate
    * @param tag The resData-tag to look for and return
    * @return The requested resData dom_ptr, or an empty dom_ptr if it cannot be found
    * @throw epp_XMLException If it is unable find the correct XML-elements in the Node
    */
   domtools::dom_ptr getresDataInfo(const domtools::dom_ptr & responseNode, const std::string & tag);
   

   /** Parses a service (svc) dom_ptr and creates the corresponding epp_Service object
    * @param node The service (svc) dom_ptr
    * @return The corresponding epp_Service object
    * @throw epp_XMLException If it is unable find the correct XML-elements in the Node
    */
   epp_Service_ref createServiceObject(const domtools::dom_ptr & node);

   /** Parses a DataCollectionService dom_ptr and creates the corresponding epp_DataCollectionPolicy object
    * @param node The DataCollectionService dom_ptr
    * @return The corresponding epp_DataCollectionPolicy object
    * @throw epp_XMLException If it is unable find the correct XML-elements in the Node
    */
   epp_DataCollectionPolicy_ref createDCPObject(const domtools::dom_ptr & node);


   /*********************************23**********************************
    *                                                                   *
    *           Only deprecated functions below this, please...         *
    *                                                                   *
    *********************************23**********************************/
   
   /** Parses an authInfo DOM_Node and creates the corresponding epp_AuthInfo object
    * @param authNode The authInfo DOM_Node
    * @return The corresponding epp_AuthInfo object
    * @deprecated Replaced by createAuthInfoObject(const domtools::dom_ptr & authNode);
    */
   epp_AuthInfo_ref createAuthInfoObject(DOM_Node authNode);
   
   /** Parses an XML epp_string and generates a DOM_Document (DOM-tree) of it
    * @param xml The XML epp_string to build a tree from
    * @return The generated DOM_Document
    * @throw epp_XMLException If it is unable to initialise the XML-parser
    * @throw epp_XMLException If it is unable to parse the epp_string
    * @deprecated Replaced by createDOMDocument(const epp_string & xml);
    */
   DOM_Document createDOM_Document(const epp_string & xml);
   
   /** Parses a DOM_Node and returns the requested resData-tag
    * @param responseNode The DOM_Node to investigate
    * @param tag The resData-tag to look for and return
    * @return The requested resData Dom_Node, or an empty DOM_Node if it cannot be found
    * @throw epp_XMLException If it is unable find the correct XML-elements in the Node
    * @deprecated Replaced by getresDataInfo(const domtools::dom_ptr & responseNode, const string & tag);
    */
   DOM_Node getresDataInfo(const DOM_Node & responseNode, const std::string & tag);
   
   /** Parses a service (svc) DOM_Node and creates the corresponding epp_Service object
    * @param node The service (svc) DOM_Node
    * @return The corresponding epp_Service object
    * @throw epp_XMLException If it is unable find the correct XML-elements in the Node
    * @deprecated Replaced by createServiceObject(domtools::dom_ptr & node);
    */
   epp_Service_ref createServiceObject(DOM_Node & node);

   /** Writes a given epp_Unspec element to the output object
    * @param outputobject The output object to write to
    * @param unspecpart The epp_Unspec element to add
    * @deprecated Replaced by addExtensionElement()
    */
   void addUnspecElement(EPP_output & outputobject,
			 const epp_Unspec_ref & unspecpart);
   
}}

#endif
