/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#ifndef __EPP_TRANSPORTSSL_H
#define __EPP_TRANSPORTSSL_H

/** @file epp_TransportSSL.h
 *  @brief Definition of epp_TransportSSL
 *  @author Asbjorn Steira Mikkelsen
 */

#include "../ssl/npssl.h"
#include <string>
#include "epp_TransportConn.h"


namespace epptransport {
   
   /// Class defining the SSL transport, which reads from and
   /// writes to a socket through a secure socket layer
   /// @see epp_TransportConn for additional information
   class epp_TransportSSL : public epp_TransportConn {
      
    private:
      std::string m_certFile; /**< SSL certificate file location */
      std::string m_cacertFile; /**< Certificate authority filename */
      std::string m_cacertDir; /**< Certificate authority directory */
      
      npbase::npssl *sslserver; /**< SSL server handler */
      int sockfd; /**< Socket file descriptor */
      
    public:
      
      /// Null constructor, setting epp_TransportBase#m_type
      epp_TransportSSL()
	{
	   m_type="epp_TransportSSL";
	   sslserver = (npbase::npssl *)NULL;
	   sockfd = 0;
	   // Default cert-file:
	   m_certFile = "/usr/share/ssl/certs/client.pem";
	};

      /// Constructor with certificate file locations
      epp_TransportSSL(const std::string & certFile,
		       const std::string & cacertFile = "",
		       const std::string & cacertDir = "")
	{
	   m_type="epp_TransportSSL";
	   sslserver = (npbase::npssl *)NULL;
	   sockfd = 0;
	   m_certFile = certFile;
	   m_cacertFile = cacertFile;
	   m_cacertDir = cacertDir;
	};
      
      /// Destructor
      virtual ~epp_TransportSSL()
	{
	   // To avoid the destructor from throwing an exception:
	   try {
	      delete sslserver;
	   } catch(...) {
	   }
	};

      /// Writing data to server
      /// @param xml_string Data to write to server
      /// @throw epp_TrException If it is not connected to the server
      /// @throw epp_TrException If it is unable write to server
      void writeToServer(const std::string & xml_string);
      
      /// Reading data from server
      /// @return Data read from server
      /// @throw epp_TrException If it is not connected to the server
      /// @throw epp_TrException If it is unable to read from server
      std::string readFromServer();
      
      /// Connecting to server by setting server name and server port and then calling #connect()
      /// @param serverName Name of server
      /// @param serverPort Port of server
      void connect(const std::string & serverName, const unsigned long & serverPort);
      
      /// Connecting to server by setting server name/port and certificate files and then calling #connect()
      /// @param serverName Name of server
      /// @param serverPort Port of server
      /// @param certFile SSL certificate file location
      /// @param cacertFile Certificate authority filename
      /// @param cacertDir Certificate authority directory
      void connect(const std::string & serverName,
		   const unsigned long & serverPort,
		   const std::string & certFile,
		   const std::string & cacertFile = "",
		   const std::string & cacertDir = "");
      
      /// Connects to server
      /// @throw epp_TrException If it is unable to connect to server
      /// @throw epp_TrException If it is unable to set up a connection with the server
      /// @throw epp_TrException If SSL handshake fails
      void connect();
      
      /// Disconnects from server
      void disconnect();
      
      
      /// Returning the location of the SSL certificate file
      /// @return Location of the SSL certificate file
      std::string getCertFile() const
	{
	   return m_certFile;
	};
      
      /// Setting the location of the SSL certificate file
      /// @param certFile Location of the SSL certificate file
      void setCertFile(const std::string & certFile)
	{
	   m_certFile = certFile;
	}
      
      /// Returning the filename of the certificate authority file
      /// @return Filename of the certificate authority file
      std::string getCACertFile() const
	{
	   return m_cacertFile;
	}
      
      /// Setting the filename of the certificate authority file
      /// @param cacertFile Filename of the certificate authority file
      void setCACertFile(const std::string & cacertFile)
	{
	   m_cacertFile = cacertFile;
	}
      
      /// Returning the directory of the certificate authority file
      /// @return Directory of the certificate authority file
      std::string getCACertDir() const 
	{
	   return m_cacertDir;
	}
      
      /// Setting the directory of the certificate authority file
      /// @param cacertDir Directory of the certificate authority file
      void setCACertDir(const std::string & cacertDir)
	{
	   m_cacertDir = cacertDir;
	}
      
      /// Tells whether the client is connected to the server or not
      /// @return True if client is connect to server, otherwise false
      bool connected() const
	{
	   return (sslserver ? true : false);
	}
   };
   
   /// Typedef for the ref to the epp_TransportSSL class
   typedef refcnt_ptr<epp_TransportSSL> epp_TransportSSL_ref;
   
}

#endif
